require 'spec_helper'
require 'soar_aspects'
require './spec/support/auditor'
require './lib/auditing/middleware'

describe "Auditing Middleware" do
  before :each do
    Auditing::Middleware.send(:public, *Auditing::Middleware.private_instance_methods)

    # Requirements to use
    # 1. Auditor
    configuration = { "preprefix" => "very important:" }
    auditor = SanityAuditor.new
    auditor.configure(configuration)
    auditor.set_audit_level(:error)

    # 2. Soar Aspects
    SoarAspects::Aspects::auditing = auditor

    # 3. Some Error Message or Standard Error
    class TestError < StandardError; end
    @error_message = 'Some error'
    @error = TestError.new(@error_message)

    # or
    @string_error = 'This is a string message not an exception'

    # or
    @standard_error_message = 'Random Reason'
    @standard_error = StandardError.new(@standard_error_message)
  end

  context "logging audit information" do
    it 'should use an auditor if configured' do
      exception = set_backtrace
      expect(SoarAspects::Aspects::auditing).to receive(:error).with(exception)
      Auditing::Middleware.error(exception)
    end

    it 'should print to STDERR if an auditor is not configured' do
      SoarAspects::Aspects::auditing = nil
      expect(STDERR).to receive(:puts).with(@error_message)
      Auditing::Middleware.error(@error_message)
    end

    it 'should support Exceptions or plain string objects' do
      SoarAspects::Aspects::auditing = nil
      expect(STDERR).to receive(:puts).with(@string_error)
      Auditing::Middleware.error(@string_error)
    end

    it 'should support error audit level' do
      expect(Auditing::Middleware.respond_to?(:error)).to eq true
    end

    it 'should support info audit level' do
      expect(Auditing::Middleware.respond_to?(:info)).to eq true
    end

    it 'should support info audit level' do
      expect(Auditing::Middleware.respond_to?(:warn)).to eq true
    end

    it 'should support debug audit level' do
      expect(Auditing::Middleware.respond_to?(:debug)).to eq true
    end

    it 'should support fatal audit level' do
      expect(Auditing::Middleware.respond_to?(:debug)).to eq true
    end

  end

  context "error handling" do
    it 'should never raise an exception so as to interfere with middleware processing' do
      allow(Auditing::Middleware).to receive(:log_to_location).and_raise(@standard_error)
      expect{Auditing::Middleware.error(@error)}.to_not raise_error
    end

    it 'should return an error message' do
      allow(Auditing::Middleware).to receive(:log_to_location).and_raise(@standard_error)
      expect(Auditing::Middleware.error(@error).message).to eq("Unexpected exception: #{@standard_error_message}")
    end
  end
end

def set_backtrace
  @error.set_backtrace(['This is a backtrace', 'Ruby rocks'])
end
